const Ad = require('../models/Ad');

// Méthode pour ajouter une nouvelle annonce
exports.addAd = function (body) {
  return new Promise(async (resolve, reject) => {
    try {
      const newAd = new Ad(body);
      const savedAd = await newAd.save();
      resolve(savedAd);
    } catch (error) {
      reject(error);
    }
  });
}

// Méthode pour mettre à jour une annonce
exports.updateAd = function (announcementId, reqBody) {
  return new Promise(async (resolve, reject) => {
    try {
      const { title, propertyType, publicationStatus, propertyStatus, description, price, availabilityDate } = reqBody;
      /*
      let photos = [];
      const oldPhotos = reqBody.oldPhotos || [];

      reqFiles.forEach(file => {
        photos.push(file.filename);
      });

      photos = photos.concat(oldPhotos);
      */
      const updatedAnnouncement = await Ad.findByIdAndUpdate(
          announcementId,
          {
            title,
            propertyType,
            publicationStatus,
            propertyStatus,
            description,
            price,
            availabilityDate,
            //photos,
          }
      );
      if (!updatedAnnouncement)
        return resolve({ status: 404, message: 'annonce non trouvée' });

      /*
      if (user.username !== updatedAnnouncement.userName) {
        throw new Error('Utilisateur non autorisé');
      }
      */
      resolve({ status: 200, message: 'Annonce Modifié avec succès' });
    } catch (error) {
      reject(error);
    }
  });
}

exports.deleteAd = function (announcementId) {
  return new Promise(async (resolve, reject) => {
    try {

      const deletedAnnouncement = await Ad.findByIdAndRemove(announcementId);

      if (!deletedAnnouncement) {
        return resolve({ status: 404, message: 'annonce non trouvée' });
      }
      /*
      if (username !== deletedAnnouncement.userName) {
        return resolve({ status: 403, message: 'User non autorisé' });
      }
      */
      /*
      deletedAnnouncement.photos.forEach((photo) => {
        const filePath = path.join('public', 'uploads', photo);
        if (fs.existsSync(filePath)) {
          fs.unlinkSync(filePath);
        }
      });
      */
      resolve({ status: 200, message: 'Annonce supprimée avec succès' });
    } catch (error) {
      reject(error);
    }
  });
};

exports.getAds = function (user) {
  return new Promise(async (resolve, reject) => {
    try {
      let announcements;
      /*
      if (user && user.isAgent) {
        announcements = await Ad.find({ userName: user.username });
      } else {
        announcements = await Ad.find({ publicationStatus: 'Publiée' });
      }
      */
      announcements = await Ad.find({ publicationStatus: 'Publiée' });
      resolve(announcements);
    } catch (error) {
      reject(error);
    }
  });
};

exports.getAdById = function (announcementId, user) {
  return new Promise(async (resolve, reject) => {
    try {
      const announcementDetails = await Ad.findById(announcementId);
      /*
      if (
          (user && user.username !== announcementDetails.userName || !user) &&
          announcementDetails.publicationStatus !== 'Publiée'
      ) {
        return resolve({ status: 404, message: 'Cette annonce est introuvable ou non disponible.' });
      }
      if (user && user.username !== announcementDetails.userName && user.isAgent) {
        return resolve({ status: 403, message: 'User non autorisée.' });
      }
      */
      if( !announcementDetails ) {
        return resolve({ status: 404, message: 'Cette annonce est introuvable ou non disponible.' });
      }
      resolve(announcementDetails);
    } catch (error) {
      reject(error);
    }
  });
};

// Service pour poser une question sur une annonce
exports.askQuestion = async function(announcementId, question, user) {
  return new Promise(async (resolve, reject) => {
      try {
      const announcement = await Ad.findById(announcementId);
      if (!announcement) {
        return resolve({ status: 404, message: 'Annonce non trouvée.' });
      }
      if (announcement.propertyStatus !== 'Disponible') {
        return resolve({ status: 404, message: 'Vous ne pouvez poser une question que sur les annonces non disponibles.' });
      }
      announcement.questions.push({
        user: "fifi",
        question
      });
      await announcement.save();
      resolve({ status: 200, message: 'Question posée avec succès.' });
    } catch (error) {
      throw error;
    }
  });
};

// Service pour répondre à une question sur une annonce
exports.answerQuestion = async function(announcementId, questionId, answer, user) {
  return new Promise(async (resolve, reject) => {
      try {
      const announcement = await Ad.findById(announcementId);
      if (!announcement) {
        return resolve({ status: 404, message: 'Annonce non trouvée.' });
      }
      /*
      if (user.username !== announcement.userName) {
        throw new Error('Utilisateur non autorisé.');
      }
       */
      const question = announcement.questions.id(questionId);
      if (!question) {
        return resolve({ status: 404, message: 'Question non trouvée.' });
      }
      question.answers.push({ user: "hajar", answer });
      await announcement.save();
      resolve({ status: 200, message: 'Réponse ajoutée avec succès.' });
    } catch (error) {
      throw error;
    }
  });
}
